@@ -40,22 +40,39 @@ class CGSmartImage extends CGExtensions
   public function __construct()
   {
     parent::__construct();
-    $this->RegisterModulePlugin();
+    $this->RegisterModulePlugin(); // this is here so the module can be used from the admin.
   }
 
+  function LazyLoadAdmin() { return TRUE; }
+  function LazyLoadFrontend() { return FALSE; }
+  function GetName() { return get_class($this); }
+  function GetFriendlyName() { return $this->Lang('friendlyname');  }
+  function GetVersion() { return '1.9'; }
+  function MinimumCMSVersion() { return '1.10.3'; }
+  function GetDependencies() { return array('CGExtensions'=>'1.26.4');   }
+  function GetAuthor() { return 'calguy1000'; }
+  function GetAuthorEmail() { return 'calguy1000@gmail.com'; }
+  function IsPluginModule() { return true; }
+  function GetAdminSection() { return 'extensions'; }
+  function HandlesEvents () { return false; }
+  function HasAdmin() { return true; }
+  function VisibleToAdminUser() { return $this->CheckPermission('Modify Site Preferences'); }
+  
+  
   public function InitializeFrontend()
   {
     parent::SetParameters();
+    $this->RestrictUnknownParams();
 
     // operational params (what to do, with what...)
 
     $this->SetParameterType('noautoscale',CLEAN_INT);
     $this->SetParameterType('nobcache',CLEAN_INT);
     $this->SetParameterType('noembed',CLEAN_INT);
-    $this->SetParameterType('noembed',CLEAN_INT);
     $this->SetParameterType('noauto',CLEAN_INT);
     $this->SetParameterType('norotate',CLEAN_INT);
     $this->SetParameterType('notag',CLEAN_INT);
+    $this->SetParameterType('noresponsive',CLEAN_INT);
     $this->SetParameterType('noremote',CLEAN_INT);
     $this->SetParameterType('src',CLEAN_STRING);
     $this->SetParameterType('overwrite',CLEAN_INT);
@@ -69,8 +86,11 @@ class CGSmartImage extends CGExtensions
     $this->SetParameterType('name',CLEAN_STRING);
     $this->SetParameterType('height',CLEAN_INT);
     $this->SetParameterType('width',CLEAN_INT);
+    $this->SetParameterType('max_height',CLEAN_INT);
+    $this->SetParameterType('max_width',CLEAN_INT);
     $this->SetParameterType('quality',CLEAN_INT);
     $this->SetParameterType('rel',CLEAN_STRING);
+
     $this->SetParameterType(CLEAN_REGEXP.'/src\d*/',CLEAN_STRING);
     $this->SetParameterType(CLEAN_REGEXP.'/alias\d*/',CLEAN_STRING);
     $this->SetParameterType(CLEAN_REGEXP.'/filter_\w*/',CLEAN_STRING);
@@ -88,10 +108,11 @@ class CGSmartImage extends CGExtensions
     $this->CreateParameter('noautoscale','',$this->Lang('param_noautoscale'));
     $this->CreateParameter('nobcache','',$this->Lang('param_nobcache'));
     $this->CreateParameter('noembed','',$this->Lang('param_noembed'));
-    $this->CreateParameter('noembed','',$this->Lang('param_noembed'));
     $this->CreateParameter('noauto','',$this->Lang('param_noauto'));
     $this->CreateParameter('norotate','',$this->Lang('param_norotate'));
     $this->CreateParameter('notag','',$this->Lang('param_notag'));
+    $this->CreateParameter('noresponsive','',$this->Lang('param_noresponsive'));
+    $this->CreateParameter('nobreakpoints','',$this->Lang('param_nobreakpoints'));
     $this->CreateParameter('noremote','',$this->Lang('param_noremote'));
     $this->CreateParameter('src','',$this->Lang('param_src'),false);
     $this->CreateParameter('overwrite','',$this->Lang('param_overwrite'));
@@ -127,168 +148,27 @@ class CGSmartImage extends CGExtensions
     $this->CreateParameter('filter_watermark','',$this->Lang('param_filter_watermark'));
   }
 
-  function LazyLoadAdmin() { return TRUE; }
-  function LazyLoadFrontend() { return FALSE; }
-
-  /*---------------------------------------------------------
-   GetName()
-   ---------------------------------------------------------*/
-  function GetName()
-  {
-    return get_class($this);
-  }
-
-  
-  /*---------------------------------------------------------
-   GetFriendlyName()
-   ---------------------------------------------------------*/
-  function GetFriendlyName()
-  {
-    return $this->Lang('friendlyname');
-  }
-  
-  
-  /*---------------------------------------------------------
-   GetVersion()
-   ---------------------------------------------------------*/
-  function GetVersion()
-  {
-    return '1.8.1';
-  }
-
-  
-  /*---------------------------------------------------------
-   MinimumCMSVersion()
-   ---------------------------------------------------------*/
-  function MinimumCMSVersion()
-  {
-    return '1.10.3';
-  }
-
-
-  /*---------------------------------------------------------
-   GetDependencies
-   ---------------------------------------------------------*/
-  function GetDependencies()
-  {
-    return array('CGExtensions'=>'1.26.4');
-  }
-
-
-  /*---------------------------------------------------------
-   GetHelp()
-   ---------------------------------------------------------*/
-  function GetHelp()
-  {
-    return $this->Lang('help');
-  }
-  
-
-  /*---------------------------------------------------------
-   GetAuthor()
-   This returns a string that is presented in the Module
-   Admin if you click on the "About" link.
-   ---------------------------------------------------------*/
-  function GetAuthor()
-  {
-    return 'calguy1000';
-  }
-  
-
-  /*---------------------------------------------------------
-   GetAuthorEmail()
-   This returns a string that is presented in the Module
-   Admin if you click on the "About" link. It helps users
-   of your module get in touch with you to send bug reports,
-   questions, cases of beer, and/or large sums of money.
-   ---------------------------------------------------------*/
-  function GetAuthorEmail()
-  {
-    return 'calguy1000@gmail.com';
-  }
-  
-
-  /*---------------------------------------------------------
-   GetChangeLog()
-   ---------------------------------------------------------*/
-  function GetChangeLog()
-  {
-    return @file_get_contents(dirname(__FILE__).'/changelog.html.inc');
-  }
-
-
-  /*---------------------------------------------------------
-   IsPluginModule()
-   ---------------------------------------------------------*/
-  function IsPluginModule()
-  {
-    return true;
-  }
-
-
-  /*---------------------------------------------------------
-   GetAdminSection()
-   ---------------------------------------------------------*/
-  function GetAdminSection()
-  {
-    return 'extensions';
-  }
-  
-
-  /*---------------------------------------------------------
-   GetAdminDescription()
-   ---------------------------------------------------------*/
+  function GetHelp() { return $this->Lang('help'); }
   function GetAdminDescription()
   {
     return $this->Lang('moddescription');
   }
   
-
-  /*---------------------------------------------------------
-   HasAdmin()
-   ---------------------------------------------------------*/
-  function HasAdmin()
+  function GetChangeLog()
   {
-    return true;
-  }
-  
-
-  /*---------------------------------------------------------
-   HasAdmin()
-   ---------------------------------------------------------*/
-  function VisibleToAdminUser()
-  {
-    return $this->CheckPermission('Modify Site Preferences');
-  }
-  
-
-  /*---------------------------------------------------------
-   HandlesEvents()
-   ---------------------------------------------------------*/
-  function HandlesEvents ()
-  {
-    return false;
+    return @file_get_contents(dirname(__FILE__).'/changelog.html.inc');
   }
 
-
-  /*---------------------------------------------------------
-   InstallPostMessage()
-   ---------------------------------------------------------*/
   function InstallPostMessage()
   {
     return $this->Lang('postinstall');
   }
 
-  
-  /*---------------------------------------------------------
-   UninstallPostMessage()
-   ---------------------------------------------------------*/
   function UninstallPostMessage()
   {
     return $this->Lang('postuninstall');
   }
 
-
   protected function can_smart_embed($src)
   {
     global $CMS_ADMIN_PAGE;
@@ -311,7 +191,6 @@ class CGSmartImage extends CGExtensions
       }
   }
 
-
   public function can_embed($src)
   {
     if( !is_readable($src) ) return FALSE;
@@ -361,7 +240,6 @@ class CGSmartImage extends CGExtensions
       }
   }
 
-
   public function clear_cached_files()
   {
     $config = cmsms()->GetConfig();
@@ -391,20 +269,17 @@ class CGSmartImage extends CGExtensions
     return $n_removed;
   }
 
-
   public function HasCapability($capability, $params = array())
   {
     if( $capability == 'tasks' ) return TRUE;
     return FALSE;
   }
 
-
   public function get_tasks()
   {
     return new CGSmartImage_ClearCacheTask();
   }
 
-
   public function get_cached_image_url($filename)
   {
     $config = cmsms()->GetConfig();
