@@ -1,7 +1,7 @@
 <?php
 #BEGIN_LICENSE
 #-------------------------------------------------------------------------
-# Module: CGSmartImage (c) 2011 by Robert Campbell (calguy1000@cmsmadesimple.org)
+# Module: CGSmartImage (c) 2012 by Robert Campbell (calguy1000@cmsmadesimple.org)
 #  
 #  An addon module for CMS Made Simple to allow creating image tags in a smart
 #  way to optimize performance.
@@ -40,6 +40,7 @@ class CGImage_Croptofit_Filter extends CGImageFilterBase
   private $_dest_w = 0;
   private $_dest_h = 0;
   private $_loc = 'c';
+  private $_upscale = 0;
   static private $_valid_locs = array('tl','tc','tr','cl','c','cr','bl','bc','br');
 
     
@@ -63,6 +64,9 @@ class CGImage_Croptofit_Filter extends CGImageFilterBase
 	  {
 	    $this->_loc = $input['loc'];
 	  }
+	if( isset($input['upscale']) ) {
+	  $this->_upscale = (int)$input['upscale'];
+	}
       }
     else if( is_array($input)  )
       {
@@ -70,13 +74,20 @@ class CGImage_Croptofit_Filter extends CGImageFilterBase
 	  {
 	    $this->_dest_w = (int)trim($input[0]);
 	    $this->_dest_h = (int)trim($input[1]);
-	    if( count($input) >= 3 && in_array($input[2],self::$_valid_locs) )
-	      {
-		$this->_loc = $input[2];
-	      }
+	    if( count($input) >= 3 && in_array($input[2],self::$_valid_locs) ) {
+	      $this->_loc = $input[2];
+	    }
+	    if( count($input) >= 4 ) {
+	      $this->_upscale = (int)$input[3];
+	    }
 	  }
       }
 
+    if( !$this->_upscale ) {
+      $this->_dest_w = cgsi_utils::trim_to_device('width',$this->_dest_w);
+      $this->_dest_h = cgsi_utils::trim_to_device('height',$this->_dest_h);
+    }
+
     if( $this->_dest_h <= 0 || $this->_dest_w <= 0 )
       {
 	throw new Exception('Invalid values specified for Croptofit filter constructor');
