@@ -42,6 +42,7 @@ class CGImage_Resize_Filter extends CGImageFilterBase
   private $_edge = 0;	   //edge size
   private $_percent = 0;   // scale to percent
   private $_resample = 1;  // resample, vs resize.
+  private $_upscale = 1;   // allow upscale by default.
 
   public function __construct(/* var args */)
   {
@@ -70,9 +71,14 @@ class CGImage_Resize_Filter extends CGImageFilterBase
 	  {
 	    $this->_resample = (int)$input['resample'];
 	  }
+	if( isset($input['upscale']) )
+	  {
+	    $this->_upscale = (int)$input['upscale'];
+	  }
       }
-    else if( is_array($input) && count($input) >= 2 && count($input) <= 3 )
+    else if( is_array($input) && count($input) >= 2 )
       {
+	$idx = 2;
 	switch( $input[0] )
 	  {
 	  case 'p':
@@ -89,10 +95,21 @@ class CGImage_Resize_Filter extends CGImageFilterBase
 	  case 'c':
 	    $this->_width = (int)$input[1];
 	    $this->_height = (int)$input[2];
+	    $idx = 3;
 	    break;
 	  }
+
+	if( count($input) >= $idx ) {
+	  $this->_upscale = (int)$input[$idx];
+	}
       }
 
+    if( $this->_width > 0 ) {
+      $this->_width = cgsi_utils::trim_to_device('width',$this->_width);
+    }
+    if( $this->_height > 0 ) {
+      $this->_height = cgsi_utils::trim_to_device('height',$this->_height);
+    }
     if( !($this->_percent > 0 || $this->_width > 0 || $this->_height > 0 || $this->_edge > 0) )
       {
 	throw new Exception('Could not Create Resize Filter - Invalid Parameters');
@@ -102,6 +119,13 @@ class CGImage_Resize_Filter extends CGImageFilterBase
 
   private function _transform(CGImageBase $src,$new_w,$new_h)
   {
+    if( !$this->_upscale ) {
+      if( $new_w > $src['width']  || $new_h > $src['height'] ) {
+	// upscaling
+	return $src;
+      }
+    }
+
     $_dest = new CGImageBase(array($src['type'],$new_w,$new_h));
 
     $res = '';
@@ -149,11 +173,13 @@ class CGImage_Resize_Filter extends CGImageFilterBase
     //find the longest edge of image and resize based on it
     if($src['width'] > $src['height']){
         $new_w = $this->_edge;
+	$new_w = cgsi_utils::trim_to_device('width',$new_w);
         $new_h = (int)($this->_edge * $src['height'] / $src['width']);
         return $this->_transform($src,$new_w,$new_h);
 
     }else{
         $new_h = $this->_edge;
+	$new_h = cgsi_utils::trim_to_device('height',$new_h);
         $new_w = (int)($this->_edge * $src['width'] / $src['height']);
         return $this->_transform($src,$new_w,$new_h);
     }
