@@ -1,7 +1,7 @@
 <?php
 #BEGIN_LICENSE
 #-------------------------------------------------------------------------
-# Module: CGSmartImage (c) 2011 by Robert Campbell (calguy1000@cmsmadesimple.org)
+# Module: CGSmartImage (c) 2012 by Robert Campbell (calguy1000@cmsmadesimple.org)
 #  
 #  An addon module for CMS Made Simple to allow creating image tags in a smart
 #  way to optimize performance.
@@ -38,7 +38,7 @@
 class cgsi_utils
 {
   protected function __construct() {}
-
+  static private $_device_data;
 
   private static function _get_alias_options($name)
   {
@@ -223,9 +223,14 @@ class cgsi_utils
     $opp['notimecheck'] = 0;
     $opp['noautoscale'] = 0;
     $opp['notag'] = 0;
+    $opp['noresponsive'] = 0;
+    $opp['nobreakpoints'] = 0;
+    $opp['max_width'] = '';
+    $opp['max_height'] = '';
     $opp['src'] = '';
     $opp['quality'] = 75;
     $opp['filters'] = array();
+    $srcimgsize = ''; // src image size
 
     $tmp = $mod->GetPreference('aliases');
     if( $tmp ) $aliases = unserialize($tmp);
@@ -309,6 +314,11 @@ class cgsi_utils
 	    $outp[$key] = (int)$value;
 	    break;
 
+	  case 'max_width':
+	  case 'max_height':
+	    $opp[$key] = (int)$value;
+	    break;
+
 	  case 'src':
 	    $opp[$key] = trim($value);
 	    break;
@@ -321,6 +331,8 @@ class cgsi_utils
 	  case 'overwrite':
 	  case 'notag':
 	  case 'noremote':
+	  case 'noresponsive':
+	  case 'nobreakpoints':
 	  case 'nobcache':
 	  case 'noembed':
 	  case 'norotate':
@@ -356,9 +368,13 @@ class cgsi_utils
       }
     if( !$outp['error'] && !$srcfile && startswith($opp['src'],'/') )
       {
-	$tmp = $config['root_path'].$opp['src'];
+	$tmp = cms_join_path($config['root_path'],$opp['src']);
 	if( file_exists($tmp) ) $srcfile = $tmp;
       }
+    if( !$outp['error'] && !$srcfile ) {
+      $tmp = cms_join_path($config['uploads_path'],$opp['src']);
+      if( file_exists($tmp) ) $srcfile = $tmp;
+    }
     if( !$outp['error'] && !$srcfile && isset($config['ssl_url']) && startswith($opp['src'],$config['ssl_url']) )
       {
 	$tmp = str_replace($config['ssl_url'],$config['root_path'],$opp['src']);
@@ -369,25 +385,32 @@ class cgsi_utils
        {
 	 // okay, gotta assume that ths is a remote file
 	 // get it, and cache it.
-	 if( file_exists($opp['src']) )
+	 $cachefile = TMP_CACHE_LOCATION.'/cgsi_'.md5($opp['src']).'.img';
+	 if( !file_exists($cachefile) )
 	   {
-	     $cachefile = TMP_CACHE_LOCATION.'/cgsi_'.md5($opp['src']).'.img';
-	     if( !file_exists($cachefile) )
-	       {
-		 $data = file_get_contents($opp['src']);
-		 if( !$data ) $opp['error'] = $mod->Lang('error_remotefile',$opp['src']);
-		 if( $data ) file_put_contents($cachefile,$data);
-	       }
-	     $srcfile = $cachefile;
+	     $data = file_get_contents($opp['src']);
+	     if( !$data ) $opp['error'] = $mod->Lang('error_remotefile',$opp['src']);
+	     if( $data ) file_put_contents($cachefile,$data);
 	   }
+	 $srcfile = $cachefile;
        }
-    if( !$outp['error'] && !$srcfile )
-      {
+    if( !$outp['error'] && !$srcfile ) {
+      $outp['error'] = $mod->Lang('error_srcnotfound',$opp['src']);
+    }
+
+    // get the source image size
+    if( !$outp['error'] ) {
+      $tmp = getimagesize($srcfile);
+      if( !is_array($tmp) || count($tmp) < 2 ) {
 	$outp['error'] = $mod->Lang('error_srcnotfound',$opp['src']);
       }
-
+      else {
+	$srcimgsize = array('width'=>$tmp[0],'height'=>$tmp[1]);
+      }
+    }
+    
     // are we automagically rotating.
-    if( !$opp['norotate'] && function_exists('exif_read_data') )
+    if( !$outp['error'] && !$opp['norotate'] && function_exists('exif_read_data') )
       {
 	// if there is already a rotate filter in the list, we won't override that.
 	$fn = 0;
@@ -448,66 +471,145 @@ class cgsi_utils
 	  }
       }
 
-    // now do some intelligence, and see if the width, and height parameters passed in
+    // now do some intelligence, and see if the width, and height arguments passed in
     // match the image size... if not, add a filter.
     if( !$outp['error'] && ($outp['width'] > 0 || $outp['height'] > 0) && $opp['noautoscale'] == 0 )
       {
-	$tmp = getimagesize($srcfile);
-	if( is_array($tmp) )
-	  {
-	    if( $outp['width'] != $tmp[0] || $outp['height'] != $tmp[1] )
-	      {
-		// we wanna display the file at a different dimenion.
-		$args = '';
-		$filter = '';
-		if( $outp['width'] && $outp['height'] )
-		  {
-		    // gonna do a crop to fit
-		    $filter = 'CGImage_Croptofit_Filter';
-		    $args = array((int)$outp['width'],(int)$outp['height']);
-		  }
-		else if( $outp['width'] )
-		  {
-		    // gonna do a resize
-		    $filter = 'CGImage_Resize_Filter';
-		    $args = array('w',(int)$outp['width']);
-		  }
-		else if ( $outp['height'] )
-		  {
-		    // gonna do a resize
-		    $filter = 'CGImage_Resize_Filter';
-		    $args = array('h',(int)$outp['height']);
-		  }
-		if( is_array($args) && $filter != '' )
-		  {
-		    $found = 0;
-		    for( $i = 0; $i < count($opp['filters']); $i++ )
-		      {
-			if( $opp['filters'][$i][0] == 'CGImage_Resize_Filter' )
-			  {
-			    $found = 1;
-			    break;
-			  }
-		      }
-		    if( !$found )
-		      {
-			$opp['filters'][] = array($filter,$args);
-		      }
-		  }
-	      }
-	  }
+	if( $outp['width'] != $srcimgsize['width'] || $outp['height'] != $srcimgsize['height'] ) {
+	  // we wanna display the file at a different dimenion.
+	  $args = '';
+	  $filter = '';
+	  if( $outp['width'] && $outp['height'] )
+	    {
+	      // gonna do a crop to fit
+	      $filter = 'CGImage_Croptofit_Filter';
+	      $args = array((int)$outp['width'],(int)$outp['height']);
+	    }
+	  else if( $outp['width'] )
+	    {
+	      // gonna do a resize
+	      $filter = 'CGImage_Resize_Filter';
+	      $args = array('w',(int)$outp['width']);
+	    }
+	  else if ( $outp['height'] )
+	    {
+	      // gonna do a resize
+	      $filter = 'CGImage_Resize_Filter';
+	      $args = array('h',(int)$outp['height']);
+	    }
+	  if( is_array($args) && $filter != '' )
+	    {
+	      $found = 0;
+	      for( $i = 0; $i < count($opp['filters']); $i++ )
+		{
+		  if( $opp['filters'][$i][0] == 'CGImage_Resize_Filter' )
+		    {
+		      $found = 1;
+		      break;
+		    }
+		}
+	      if( !$found )
+		{
+		  $opp['filters'][] = array($filter,$args);
+		}
+	    }
+	}
       }
 
+    // doing responsive images... get device width and height.
+    $is_responsive = 0;
+    if( !$outp['error'] && $mod->GetPreference('responsive',1) && $opp['noresponsive'] == 0 ) {
+      $device = self::get_device_capabilities();
+      if( is_array($device) && isset($device['width']) && isset($device['height']) ) {
+	// we found device capabilities.
+
+	// we have to do auto-scaling now, responsive stuff trumps it.
+	$opp['noautoscale'] = 0;
+	$is_responsive = 1;
+	
+	// merge that data with any max_width and max_height that have already been supplied.
+	$w = max(1,(int)$device['width']);
+	if( $opp['max_width'] > 0 ) {
+	  $opp['max_width'] = min($opp['max_width'],$w);
+	}
+	else {
+	  $opp['max_width'] = $w;
+	}
+	
+	$h = max(1,(int)$device['height']);
+	if( $opp['max_height'] > 0 ) {
+	  $opp['max_height'] = min($opp['max_height'],$h);
+	}
+	else {
+	  $opp['max_height'] = $h;
+	}
+      }
+    }
+
+    // if we have max width and/or height... add a filter here at the end
+    // to make sure that our output never exceeds those max values.
+    if( !$outp['error'] &&
+	($opp['max_width'] > 0 || $opp['max_height'] > 0) &&
+	$opp['noautoscale'] == 0 ) {
+
+      // if we have breakpoints, use find the closest one
+      $args = '';
+      $filter = '';
+      $bp = null;
+	
+      if( ($tmp = $mod->GetPreference('responsive_breakpoints')) && $is_responsive == 1 &&
+	  !$opp['nobreakpoints'] ) {
+	$tmp = explode(',',$tmp);
+	$bp = array();
+	for( $i = 0; $i < count($tmp); $i++ ) {
+	  $t1 = (int)trim($tmp[$i]);
+	  if( $t1 > 0 ) $bp[] = $t1;
+	}
+	asort($bp);
+      }
+      
+      if( is_array($bp) && count($bp) ) {
+	// we have valid breakpoints.
+	$lval = max($opp['max_width'],$opp['max_height']);
+	for( $i = count($bp)-1; $i > 0; $i-- ) {
+	  if( $bp[$i] > $lval ) continue;
+	  break;
+	}
+
+	// resize to maximum dimension of lval
+	// note, it'd be nice to have some kind of settings here
+	// i.e: which filer (and other params) to use.
+	$lval = $bp[$i];
+	$filter = 'CGImage_Resize_Filter';
+	$args = array('e',$lval,0);
+      }
+      else {
+	// standard handling of max width/max height.
+	if( $opp['max_width'] && $opp['max_height'] ) {
+	  $filter = 'CGImage_Croptofit_Filter';
+	  $args = array((int)$opp['max_width'],(int)$opp['max_height']);
+	}
+	else if( $opp['max_width'] ) {
+	  $filter = 'CGImage_Resize_Filter';
+	  $args = array('w',(int)$opp['max_width'],0);
+	}
+	else if( $opp['max_height'] ) {
+	  $filter = 'CGImage_Resize_Filter';
+	  $args = array('h',(int)$opp['max_height'],0);
+	}
+      }
+      $opp['filters'][] = array($filter,$args);
+    }
+    
     //
     // end of smartness stuff... now begin the work
     //
-    if( !$outp['error'] && count(array_keys($opp['filters'])) )
-      {
-	//
-	// we're doing some kind of magic to the image.
-	//
-	$want_transform = 1;
-      }
+    if( !$outp['error'] && count(array_keys($opp['filters'])) ) {
+      //
+      // we're doing some kind of magic to the image.
+      //
+      $want_transform = 1;
+    }
 
     if( !$outp['error'] && $srcfile && $want_transform )
       {
@@ -693,6 +795,9 @@ class cgsi_utils
     $dom = new CGDomDocument();
     $dom->strictErrorChecking = FALSE;
     $dom->validateOnParse = FALSE;
+    if( function_exists('mb_convert_encoding') ) {
+      $content = mb_convert_encoding($content, 'HTML-ENTITIES', 'UTF-8');
+    }
     $dom->loadHTML($content);
 
     $imgs = $dom->GetElementsByTagName('img');
@@ -777,6 +882,70 @@ class cgsi_utils
       }
     return $content;
   }
+
+
+  public static function get_device_capabilities()
+  {
+    if( is_array(self::$_device_data) && isset(self::$_device_data['width']) ) {
+      return self::$_device_data;
+    }
+
+    if( !isset($_SERVER['HTTP_USER_AGENT']) ) return;
+    $browser = cge_utils::get_browser();
+    if( !$browser->isMobile() ) return;
+
+    $agent = $_SERVER['HTTP_USER_AGENT'];
+    $config = cmsms()->GetConfig();
+    $ckey = md5($config['root_url'].'CGExtensionsDeviceCapabilitiesCache');
+
+    $handler = cms_cache_handler::get_instance();
+    $data = array();
+    if( $handler->exists($ckey) ) {
+      $rawdat = $handler->get($ckey);
+      if( $rawdat ) {
+	$data = unserialize($rawdat);
+      }
+    }
+    $ukey = md5($agent);
+    if( isset($data[$ukey]) ) {
+      self::$_device_data = $data[$ukey];
+      return $data[$ukey];
+    }
+    
+    // gotta do a web service request.
+    // here we would determine what client to use.
+    $mod = cms_utils::get_module('CGSmartImage');
+    if( !$mod ) return;
+
+    // instantiate the class and do the request
+    $client_class = 'cgsi_DeviceAtlasPersonal_client';
+    if( !class_exists($client_class) ) {
+      // no client class found.
+      return;
+    }
+    $client_obj = new $client_class($agent);
+    if( !is_object($client_obj) ) {
+      // no client class possible.
+      return;
+    }
+    $res = $client_obj->get_device_resolution();
+    if( !is_array($res) ) return;
+
+    // got results from web service
+    $data[$ukey] = $res;
+    $handler->set($ckey,serialize($data));
+    self::$_device_data = $res;
+    return $res;
+  }
+
+  public static function trim_to_device($flag,$value)
+  {
+    $flag = strtolower($flag);
+    if( $value > 0 && is_array(self::$_device_data) && isset(self::$_device_data[$flag]) ) {
+      return min($value,self::$_device_data[$flag]);
+    }
+    return $value;
+  }
 }
 
 #
